#!/bin/bash


accumsize=$(( $1 ))
if [ $accumsize -lt 8 ] || [ $accumsize -gt 32 ]; then
	printf "\n Usage: $0 <algorithm-word-size> <algorithm>\n Word-size range is 8-32\n"
	exit
fi

if [ ! -f "src/algorithmlist.cfg" ]; then
	echo Aborted!  src/algorithmlist.cfg missing
	echo
	exit
fi
read eng engname hashname line <<<$( grep -Fw "$2" "src/algorithmlist.cfg" )
if [ "$eng" != "$2" ]; then
	printf "\n Usage: $0 <algorithm-word-size> <algorithm>\n Specify PRNG algorithm from first column of list:\n"
	cat "src/algorithmlist.cfg"
	exit
fi
enginetitle="${engname}$(($accumsize*2))(${accumsize})${hashname}"
enginedir="tests/engine-${eng}-s${accumsize}"
testset="${enginedir}/maxiter-${eng}-s${accumsize}.out"

if [ ! -d "tests" ]; then
	mkdir "tests"
fi
if [ ! -d "$enginedir" ]; then
	mkdir "$enginedir"
fi



function  fullperiod_culling
{
	printf "\n\n__________________________________________________________________________________________________________\n    BEGIN  ... at "; date +"%F %T"
	printf "\n   Launching tasks to brute force find full-period candidates for the\n   back-end engine of $enginetitle pseudo-random number generator:\n"

	if [ -f "$testset" ]; then
		echo Aborted!  Previously generated test-set, $testset, already present.  Erase first
		echo
		exit
	fi
	if [ $accumsize -lt 17 ]; then
		statuson=""
	else
		statuson="-DSTATUS_ON"
	fi
	# Check for spare mips - initial sample point
	activeafter=0
	prevload=0
	read cpu user nice system idle iowait irq softirq steal line <"/proc/stat"
	activebefore=$(( $user + $nice + $system + $softirq + $steal ))
	totalbefore=$(( $activeafter + $idle + $iowait ))

	case "$eng" in
	xon)
		m=$((-1))
		;;
	xofb)
		m=$((0x46F5))
		;;
	*)
		m=$((0))
		;;
	esac
	top=$(( $accumsize-1 ))

for b in `seq 1 $top`; do
for a in `seq 1 $top`; do
for c in `seq $a $top`; do

	RNGbin="${enginedir}/maxiter-${eng}-s${accumsize}-a${a}b${b}c${c}"

	(printf "Testing ${eng}-s${accumsize}-a${a}b${b}c${c} at "; date +"%F %T")

	# Compile test case
	gcc -o "$RNGbin" "src/xoro-fullperiod64.c" -O3 -DACCUM_SIZE=$accumsize -DCONSTANT_A=$a -DCONSTANT_B=$b -DCONSTANT_C=$c -DCONSTANT_M=$m $statuson

	if [ $? -ne 0 ]; then
		echo $enginetitle test case, $RNGbin aborted!  Compile error
		echo
		exit
	fi

	# Run test case concurrently with others
	nice -12 "$RNGbin" "$testset" &

	# Check for spare mips
	while true
	do
		sleep 0.1

		read cpu user nice system idle iowait irq softirq steal line <"/proc/stat"
		activeafter=$(( $user + $nice + $system + $softirq + $steal ))
		totalafter=$(( $activeafter + $idle + $iowait ))

		currload=$(( 101 * ($activeafter - $activebefore) / ($totalafter - $totalbefore) ))
		activebefore=$activeafter
		totalbefore=$totalafter

		cpuload=$(( $currload > $prevload ? $currload : $prevload ))  # max()
		prevload=$currload

		if [ $cpuload -lt 80 ]; then  # Suited to dual threaded processor cores
			break
		fi
#		printf "cpuload=${currload}  \r" >&2

		sleep 1
	done

	rm "${RNGbin}"	# Delete the binary, even though tests are still running
done
done
done
}


fullperiod_culling |tee -a "${enginedir}/maxiter-${eng}-s${accumsize}.log"

