#!/bin/bash
diter=""		# single iteration per sample
diter="-DdITER"	# uncomment for double iteration per sample


PRoptions="stdin -te 1 -tf 2 -tlmin 1KB"
#PRoptions="stdin -multithreaded -te 1 -tf 2 -tlmin 1KB -tlmaxonly"
PRcommand="./RNG_test.unchanged ${PRoptions}"
#PRcommand="./RNG_test.FPFless ${PRoptions}"



accumsize=$(( $1 ))
if [ $accumsize -lt 8 ] || [ $accumsize -gt 32 ]; then
	printf "\n Usage: $0 <algorithm-word-size> <algorithm>\n Word-size range is 8-32\n"
	exit
fi

if [ ! -f "src/algorithmlist.cfg" ]; then
	echo Aborted!  src/algorithmlist.cfg missing
	echo
	exit
fi
read eng engname hashname line <<<$( grep -Fw "$2" "src/algorithmlist.cfg" )
if [ "$eng" != "$2" ]; then
	printf "\n Usage: $0 <algorithm-word-size> <algorithm>\n Specify PRNG algorithm from first column of list:\n"
	cat "src/algorithmlist.cfg"
	exit
fi
enginetitle="${engname}$(($accumsize*2))(${accumsize})${hashname}"
enginedir="tests/engine-${eng}-s${accumsize}"
candidateslist="${enginedir}/candidates-${eng}-s${accumsize}${diter:2:1}.cfg"
#eng=${enginename[$(($engine*3))]}
#enginetitle="${enginename[$(($engine*3+1))]}$(($accumsize*2))(${accumsize})${enginename[$(($engine*3+2))]}"
#enginesource="src/${enginename[$(($engine*3+1))],,}128${enginename[$(($engine*3+2))]}"  # ${,,} forces lower case

if [ ! -d "tests" ]; then
	mkdir "tests"
fi
if [ ! -d "$enginedir" ]; then
	mkdir "$enginedir"
fi
if [ ! -f "$candidateslist" ]; then
	echo Aborted!  $enginetitle candidate set, $candidateslist, missing
	echo
	exit
fi



function  cpu_wait {
# Check for spare mips
	while true
	do
		sleep 0.5

		read cpu user nice system idle iowait irq softirq steal guest <"/proc/stat"
		activeafter=$(( $user + $nice + $system + $softirq + $steal ))
		totalafter=$(( $activeafter + $idle + $iowait ))

		currload=$(( 101 * ($activeafter - $activebefore) / ($totalafter - $totalbefore) ))
		activebefore=$activeafter
		totalbefore=$totalafter

		cpuload=$(( $currload > $prevload ? $currload : $prevload ))  # max()
		prevload=$currload

		if [ $cpuload -lt 48 ]; then  # Suited to dual threaded processor cores
			break
		fi
#		printf "cpuload=${currload}  \r" >&2

		sleep 0.5
	done
}


function  practrand_scoring {
(printf "\n\n__________________________________________________________________________________________________________\n    BEGIN  ... at "; date +"%F %T"; printf "PractRand v0.93 options:  ${PRoptions}\n\n")

if [ -z $diter ]; then # single iteration per sample
	testingdir="${enginedir}/pract"
	string3="single"
	top=$(( $accumsize - 1 ))
	stepsize=1
else # double iteration per sample
	testingdir="${enginedir}/pract-diter"
	string3="double"
	top=$(( $accumsize * 2 - 1 ))
	stepsize=2
fi
if [ ! -d "$testingdir" ]; then
	mkdir "$testingdir"
fi
# Compile Xoroshiron correction algorithm
gcc -o "${enginedir}/xon-corr" "src/xon-correction.c" -O3 -DACCUM_SIZE=$accumsize -DSTATUS_ON

while IFS=", ]" read a b c d line
do
	a=$(($a));b=$(($b));c=$(($c));d=$(($d))
	if [ $a -eq 0 ] || [ $b -eq 0 ] || [ $c -eq 0 ]; then
		sampsize=0
	else
		case "$eng" in
		xon)
			m=$((-1))
			e=$(("$( "${enginedir}/xon-corr" $a $b $c $d $m )"))
			string4=", correction ${e}:${m},"
			;;
		xofb)
			m=$((0x46F5))
			e=$(("$( "${enginedir}/xon-corr" $a $b $c $d $m )"))
			string4=", correction ${e}:${m},"
			;;
		*)
			m=$((0))
			e=$((0))
			string4=""
			;;
		esac
		printf "\nGrid scoring for candidate [${a} ${b} ${c} ${d}]${string4} of ${string3} iterated ${enginetitle} random generator\n"

		read cpu user nice system idle iowait irq softirq steal guest <"/proc/stat"
		activebefore=$(( $user + $nice + $system + $softirq + $steal ))
		totalbefore=$(( $activeafter + $idle + $iowait ))
		prevload=0

		scoredir="${testingdir}/a${a}b${b}c${c}"
		if [ ! -d "$scoredir" ]; then
			mkdir "$scoredir"
		fi
		sampsize=$(( $top + 1 ))
	fi
#	while [ $sampsize -gt 0 ]
	while [ $sampsize -ge 4 ]
	do
		samppos=0
		while [ $samppos -le $top ]
		do
			(printf "  Sampling aperture ${sampsize}>>${samppos}  at "; date +"%F %T")

			RNGbin="${scoredir}/case-a${a}b${b}c${c}d${d}w${sampsize}p${samppos}"
			PRreport="${scoredir}/PRscore-samp${sampsize}+${samppos}${diter:2:1}-a${a}b${b}c${c}d${d}.out"

			# Compile candidate generator
			gcc -o "${enginedir}/prnggrid.o" -c "src/${engname,,}128${hashname,,}.c" -O3 -DACCUM_SIZE=$accumsize -DCONSTANT_A=$a -DCONSTANT_B=$b -DCONSTANT_C=$c -DCONSTANT_D=$d -DCONSTANT_E=$e -DCONSTANT_M=$m
			if [ $? -ne 0 ]; then
				echo $enginetitle test case aborted!  Compile error
				echo
				exit 2
			fi
			# Compile candidate test case
			gcc -o "$RNGbin" "${enginedir}/prnggrid.o" "src/xoro128-test.c" -O3 -DACCUM_SIZE=$accumsize -DCONSTANT_A=$a -DCONSTANT_B=$b -DCONSTANT_C=$c -DCONSTANT_D=$d -DCONSTANT_E=$e -DCONSTANT_M=$m -DSAMPLE_SIZE=$sampsize -DSAMPLE_POSITION=$samppos -DENGINETITLE=\"${enginetitle}\" $diter
			if [ $? -ne 0 ]; then
				echo $enginetitle test case aborted!  Compile error
				echo
				exit 1
			fi

			nice -11  src/score-one.sh "$RNGbin" "$PRcommand" "$PRreport" &
			cpu_wait
			samppos=$(( $samppos + 1 ))
		done
#		sampsize=$(( $sampsize - $stepsize ))
		sampsize=$(( $sampsize / 2 ))
		if [ $sampsize -gt 4 ] && [ $(( $sampsize / 8 * 8 )) -ne $sampsize ]; then
			sampsize=$(( ($sampsize / 8 + 1) * 8 ))  # round up to multiple of 8
		fi
	done
done < "$candidateslist"

rm "${enginedir}/prnggrid.o"
rm "${enginedir}/xon-corr"
}


practrand_scoring |tee -a "${enginedir}/PRscore-${eng}-s${accumsize}.log"

