#include <limits.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>
#include <time.h>


//#define  ACCUM_SIZE  16   // Passed in as -D gcc parameter. (Min=8, Max=32) Bit width of generated random numbers.

#if ACCUM_SIZE > 32
	#define  ACCUM_SIZE  32
#endif

#include "xorotypes.h"


#define  STR_HELPER(x) #x
#define  STR(x) STR_HELPER(x)



// derives constant E as some sort of correction to Xoroshiron algorithm
static rngword_t  FindECorr( int a, int b, int c, int d, rngword_t m )
{
	rngword_t  s0 = 1, s1 = 0, x = 0, y = 0;

	do {
		do {
			s1 = y ^ x;
			if( rotl( s1, c ) == y )
			{
				s0 = rotl( x, a ) ^ m ^ s1 ^ (s1 << b);
				if( s0 == x )  	return( x + rotl( x + y, d ) );
			}
			#if (ACCUM_SIZE == 16) || (ACCUM_SIZE == 32)
			x++;
			#else
			x = (x + 1) & ACCUM_MASK;
			#endif
		} while( x );

		#if (ACCUM_SIZE == 16) || (ACCUM_SIZE == 32)
		y++;
		#else
		y = (y + 1) & ACCUM_MASK;
		#endif
	} while( y );

	return( 0 );
}




int  main( int argc, char* argv[] )
{
	int  constant_a, constant_b, constant_c, constant_d;
	rngword_t  constant_e;
#ifdef STATUS_ON
	time_t  enddate;
	struct timespec  start, stop;
	double  duration;
	char  datebuff[40];
#endif


	if( argc != 6 )  {
//		fputs( "\nError!  Three constants required.\n", stderr );
		fprintf( stderr, "Error!  Five parameters, A-E, required.\n" );
		exit(1);
	}
//	printf( "\nConfirm: argv[1]=%s\n", argv[1] );
	constant_a = strtol( argv[1], NULL, 10 );
	constant_b = strtol( argv[2], NULL, 10 );
	constant_c = strtol( argv[3], NULL, 10 );
	constant_d = strtol( argv[4], NULL, 10 );
	constant_e = strtol( argv[5], NULL, 10 );

#ifdef STATUS_ON
// Mark time - initial sample point
	clock_gettime( CLOCK_PROCESS_CPUTIME_ID, &start );  // POSIX
#endif

//
// Do it!
//
	constant_e = FindECorr( constant_a, constant_b, constant_c, constant_d, constant_e );
	printf( "%d\n", constant_e );

#ifdef STATUS_ON
	clock_gettime( CLOCK_PROCESS_CPUTIME_ID, &stop );  // POSIX
	duration = (time_t)(stop.tv_sec - start.tv_sec) + (long)(stop.tv_nsec - start.tv_nsec)/1.0e9;
	enddate = time(NULL);
	strftime( datebuff, sizeof datebuff, "%T", localtime(&enddate) );
	fprintf( stderr, "Constant E for candidate [%d %d %d %d] is %d and took %.3f seconds at %s\n", constant_a, constant_b, constant_c, constant_d, constant_e, duration, datebuff );
#endif

	return( 0 );
}
