#include <limits.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>
#include <time.h>


//#define  ACCUM_SIZE  16   // Passed in as -D gcc parameter. (Min=9, Max=32) Bit width of generated random numbers.
//#define  CONSTANT_A  14   // Passed in as -D gcc parameter
//#define  CONSTANT_B  2    // Passed in as -D gcc parameter
//#define  CONSTANT_C  7    // Passed in as -D gcc parameter

#if ACCUM_SIZE > 32
	#define  ACCUM_SIZE  32
#endif

#include "xorotypes.h"



//  Find max iterations of every combination of constants in the Xoroshiro random number generator.
int  main(int argc, char* argv[])
{
	drnword_t  count;
	rngword_t  s0, s1;  // Must be static to prevent optimiser vanishing the whole iterator!
	rngword_t  work;
	FILE  *fh;
#ifdef STATUS_ON
	time_t  enddate;
	struct timespec  starttime, endtime;
	char  datebuff[40];
#endif


	if( argc != 2 )  {
//		fputs( "\nError!  Three constants required.\n", stderr );
		fprintf( stderr, "Error!  Parameters required.\n" );
		exit(1);
	}
//	printf( "\nConfirm: argv[1]=%s\n", argv[1] );

#ifdef STATUS_ON
// Mark time - initial sample point
	clock_gettime(CLOCK_PROCESS_CPUTIME_ID, &starttime);  // POSIX
#endif

//
// Do it!
//
	s0 = 1; s1 = 0;   // Seed the state array.  Seed MUST not be all zero.
	count = 0;

// Brute force iterate to find full length repeats (2^32 - 1 for 16-bit accumulators, 32-bit state)
	do {
#if (ACCUM_SIZE == 16) || (ACCUM_SIZE == 32)
		count++;
#else
		count = (count + 1) & ACCUM_DMASK;
#endif

		work = s0 ^ s1;
		s0 = (rotl( s0, CONSTANT_A ) ^ CONSTANT_M ^ work ^ (work << CONSTANT_B)) & ACCUM_MASK;
		s1 = rotl( work, CONSTANT_C );


		if( s0 == 1 && s1 == 0 )
			break;
	} while( count );

#ifdef STATUS_ON
	clock_gettime(CLOCK_PROCESS_CPUTIME_ID, &endtime);  // POSIX
	enddate = time(NULL);
	strftime( datebuff, sizeof datebuff, "%T", localtime(&enddate) );
	#if ACCUM_SIZE > 16
	fprintf( stderr, "Case [%d %d %d] iterated %lx times, using %ld seconds at %s\n", CONSTANT_A, CONSTANT_B, CONSTANT_C, count, endtime.tv_sec - starttime.tv_sec, datebuff );
	#else
	fprintf( stderr, "Case [%d %d %d] iterated %x times, using %ld seconds at %s\n", CONSTANT_A, CONSTANT_B, CONSTANT_C, count, endtime.tv_sec - starttime.tv_sec, datebuff );
	#endif

	starttime.tv_sec = endtime.tv_sec;
#endif

	if( count == ACCUM_DMASK )  {
		if( argc == 2 )  {

			printf( "Case [%d %d %d] is a full-period candidate\n", CONSTANT_A, CONSTANT_B, CONSTANT_C );
			#if CONSTANT_A != CONSTANT_C
			printf( " and [%d %d %d] assumed too\n", CONSTANT_C, CONSTANT_B, CONSTANT_A );
			#endif

			// Append good constants to candidate set file
			fh = fopen( argv[1], "a" );

			if( fh != NULL )
			{
				fprintf( fh, " %2d %2d %2d\n", CONSTANT_A, CONSTANT_B, CONSTANT_C );
				#if CONSTANT_A != CONSTANT_C
				fprintf( fh, " %2d %2d %2d\n", CONSTANT_C, CONSTANT_B, CONSTANT_A );
				#endif
				fclose( fh );
			}
			else	printf( "File error writing case [%d %d %d]\n", CONSTANT_A, CONSTANT_B, CONSTANT_C );
		}
	} else if( count == 0 )  {
		printf( "Impossible period with case [%d,%d,%d].  s0=%x s1=%x\n", CONSTANT_A, CONSTANT_B, CONSTANT_C, s0, s1 );
	}

	return 0;
}
