#include <limits.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>


//#define  ACCUM_SIZE  16       // Passed in as -D gcc parameter. (Min=8+WORD_POSITION, Max=64) Bit width of generated random numbers.
//#define  SAMPLE_POSITION  1   // Passed in as -D gcc parameter.
//#define  SAMPLE_SIZE          // Passed in as -D gcc parameter. Used to be (ACCUM_SIZE - SAMPLE_POSITION)

#ifdef SCRO
	#define  ACCUM_SIZE  32   // Fixed
	#undef   dITER
#endif

#include "xorotypes.h"


#define  STR_HELPER(x) #x
#define  STR(x) STR_HELPER(x)



static inline uint32_t  revbits( uint32_t  v )
{
    // https://graphics.stanford.edu/~seander/bithacks.html
    // swap odd and even bits
    v = ((v >> 1) & 0x55555555) | ((v & 0x55555555) << 1);
    // swap consecutive pairs
    v = ((v >> 2) & 0x33333333) | ((v & 0x33333333) << 2);
    // swap nibbles ...
    v = ((v >> 4) & 0x0f0f0f0f) | ((v & 0x0f0f0f0f) << 4);
    // swap bytes
    v = ((v >> 8) & 0x00ff00ff) | ((v & 0x00ff00ff) << 8);
    // swap 2-byte-long pairs
    v = ( v >> 16             ) | ( v               << 16);

    return v;
}



int  main( int argc, char *argv[] )
{
	uint8_t  buff[1040];
#ifdef dITER
	drnword_t  random64;
#else
	rngword_t  random64;
#endif
	int  chunk = 0, bufidx = 0;
	unsigned  k = 0;


	// Print some randomness as Little-Endian byte ordering.
	while(1)
	{
#ifdef dITER
	random64 = rotr( (drnword_t)nextrnd() | ((drnword_t)nextrnd() << ACCUM_SIZE), SAMPLE_POSITION );
#else
	random64 = rotr( nextrnd(), SAMPLE_POSITION );
#endif

#if SAMPLE_SIZE == 8
		buff[bufidx++] = random64;

#elif SAMPLE_SIZE == 16
		*(uint16_t *)&buff[bufidx] = random64;
		bufidx += 2;

#elif SAMPLE_SIZE == 24
		*(uint32_t *)&buff[bufidx] = random64;
		bufidx += 3;

#elif SAMPLE_SIZE == 32
		*(uint32_t *)&buff[bufidx] = random64;
		bufidx += 4;

#elif SAMPLE_SIZE == 40
		*(uint64_t *)&buff[bufidx] = random64;
		bufidx += 5;

#elif SAMPLE_SIZE == 48
		*(uint64_t *)&buff[bufidx] = random64;
		bufidx += 6;

#elif SAMPLE_SIZE == 56
		*(uint64_t *)&buff[bufidx] = random64;
		bufidx += 7;

#elif SAMPLE_SIZE == 64
		*(uint64_t *)&buff[bufidx] = random64;
		bufidx += 8;

#elif SAMPLE_SIZE > 8
		random64 &= SAMPLE_MASK;

		if( chunk ) {  // Any remainder?
			k |= (random64 << chunk) & 0xff;   // Concatenate to full byte (Little Endian ordering)
			chunk = 8 - chunk;   // Concatenating size
			buff[bufidx++] = k;
//			printf( "%02x ", k );
		}

		while(chunk <= (SAMPLE_SIZE - 8)) {   //  Stop at last full byte
			k = random64 >> chunk;   // One byte (Little Endian ordering)
			chunk += 8;
			buff[bufidx++] = k;
//			printf( "%02x ", k );
		}
//		printf("\n");

		k = random64 >> chunk;   // Remainder data (to be concatenated on next iteration)
		chunk = SAMPLE_SIZE - chunk;  //  Remainder size
//		printf( "%02x %04d ", k, chunk );

#else
		k |= (random64 & SAMPLE_MASK) << chunk;
		chunk += SAMPLE_SIZE;

		if( chunk >= 8 )
		{
			buff[bufidx++] = k;
			chunk -= 8;
			k >>= 8;
			
			if( bufidx >= 1024 )
			{
				fwrite( buff, bufidx, 1, stdout );
				bufidx = 0;
			}
		}
#endif
#if SAMPLE_SIZE >= 8
		if( bufidx >= 1024 )
		{
	#ifdef TEXTOUT
		#ifdef pARITY
			printf( "\n"ENGINETITLE"%d%s",ACCUM_SIZE*2 ,(CONSTANT_D>0?"+p+":"+p") );
		#else
			printf( "\n"ENGINETITLE"%d%s",ACCUM_SIZE*2 ,(CONSTANT_D>0?"++":"+") );
		#endif
			printf( "  [%d %d %d%s]\n", CONSTANT_A, CONSTANT_B, CONSTANT_C, (CONSTANT_D>0?" "STR(CONSTANT_D):"") );
		#ifdef dITER
			printf( " - Double Iterated" );
		#else
			printf( " - Single Iterated" );
		#endif
			printf( "\n - Sampling Aperture %d>>%d\n", SAMPLE_SIZE, SAMPLE_POSITION );
			printf( " 3 2 1 0  7 6 5 4  b a 9 8  f e d c\n===================================\n" );
			for( bufidx = 0; bufidx < 1024; bufidx += 16 )
			{
				printf( "%08x %08x %08x %08x\n",
					*(uint32_t *)&buff[bufidx],
					*(uint32_t *)&buff[bufidx+4],
					*(uint32_t *)&buff[bufidx+8],
					*(uint32_t *)&buff[bufidx+12] );
			}
			printf( "\n\n" );
			exit(0);
	#else
		#ifdef REVBITS
			for( bufidx = 0; bufidx < 1024; bufidx += 4 )
				*(uint32_t *)&buff[bufidx] = revbits( *(uint32_t *)&buff[bufidx] );
		#endif
			fwrite( buff, bufidx, 1, stdout );
	#endif
			bufidx = 0;
		}
#endif
	}

	return 0;
}

